<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://welaunch.io/plugins
 * @since      1.0.0
 *
 * @package    WordPress_Form_Wizard
 * @subpackage WordPress_Form_Wizard/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    WordPress_Form_Wizard
 * @subpackage WordPress_Form_Wizard/includes
 * @author     Daniel Barenkamp <support@welaunch.io>
 */
class WordPress_Form_Wizard {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      WordPress_Form_Wizard_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */

	public function __construct($version) 
	{
		$this->plugin_name = 'wordpress-form-wizard';
		$this->version = $version;

		$this->load_dependencies();
		$this->set_locale();
		$this->define_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - WordPress_Form_Wizard_Loader. Orchestrates the hooks of the plugin.
	 * - WordPress_Form_Wizard_i18n. Defines internationalization functionality.
	 * - WordPress_Form_Wizard_Admin. Defines all hooks for the admin area.
	 * - WordPress_Form_Wizard_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{
		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wordpress-form-wizard-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-wordpress-form-wizard-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-admin.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-analytics.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-forms.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-entries.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-questions.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-mailer.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-answers.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'classes/class-wordpress-form-wizard-public.php';
		

     	if ( file_exists( plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php' ) ) {
	        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'vendor/autoload.php';
	    }

		$this->loader = new WordPress_Form_Wizard_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the WordPress_Form_Wizard_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() 
	{
		$this->plugin_i18n = new WordPress_Form_Wizard_i18n();
		$this->loader->add_action( 'plugins_loaded', $this->plugin_i18n, 'load_plugin_textdomain' );
	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_hooks() 
	{
		// Admin
		$this->admin = new WordPress_Form_Wizard_Admin( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'plugins_loaded', $this->admin, 'load_redux' );
		$this->loader->add_action( 'init', $this->admin, 'init' );
		$this->loader->add_action( 'admin_enqueue_scripts', $this->admin, 'enqueue_scripts_styles', 999);

		if(isset($_GET['form-wiazard-migrate-keys'])) {

			$this->loader->add_action( 'init', $this->admin, 'migrate_keys' );
		}

		// Analytics
		$this->analytics = new WordPress_Form_Wizard_Analytics( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'init', $this->analytics, 'init' );

		$this->forms = new WordPress_Form_Wizard_Forms( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action('init', $this->forms, 'init', 20);
        $this->loader->add_action('add_meta_boxes', $this->forms, 'add_custom_metaboxes', 10, 2);
        $this->loader->add_action('save_post', $this->forms, 'save_custom_metaboxes', 1, 2);

		$this->mailer = new WordPress_Form_Wizard_Mailer( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action('init', $this->mailer, 'init', 20);
		$this->loader->add_action('wp_ajax_wordpress_form_wizard_send_mail', $this->mailer, 'send_email', 20);
		$this->loader->add_action('wp_ajax_nopriv_wordpress_form_wizard_send_mail', $this->mailer, 'send_email', 20);	

        // Questions
		$this->questions = new WordPress_Form_Wizard_Questions( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action('init', $this->questions, 'init', 30);
        $this->loader->add_action('add_meta_boxes', $this->questions, 'add_custom_metaboxes', 10, 2);
        $this->loader->add_action('save_post', $this->questions, 'save_custom_metaboxes', 1, 2);
           	

		$this->answers = new WordPress_Form_Wizard_Answers( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action('init', $this->answers, 'init', 40);
        $this->loader->add_action('add_meta_boxes', $this->answers, 'add_custom_metaboxes', 10, 2);
        $this->loader->add_action('save_post', $this->answers, 'save_custom_metaboxes', 1, 2);


    	// Public
		$this->public = new WordPress_Form_Wizard_Public( $this->get_plugin_name(), $this->get_version() );
        $this->loader->add_action( 'wp_enqueue_scripts', $this->public, 'enqueue_scripts_styles');  
        $this->loader->add_action( 'init', $this->public, 'init');  

        // Entries
		$this->entries = new WordPress_Form_Wizard_Entries( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action('init', $this->entries, 'init', 20);

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() 
	{
		$this->loader->run();
	}

	/**
	 * Get Options
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @param   [type]                       $option [description]
	 * @return  [type]                               [description]
	 */
    protected function get_option($option)
    {
    	if(!isset($this->options)) {
    		return false;
    	}

    	if(!is_array($this->options)) {
    		return false;
    	}
    	
    	if(!array_key_exists($option, $this->options)) {
    		return false;
    	}

    	return $this->options[$option];
    }

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    WordPress_Form_Wizard_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

 	protected function get_user_order_count($userId = 0) 
 	{
	    global $wpdb;
	    
	    // Based on user ID (registered users)
	    if ( is_numeric( $userId) ) { 
	        $meta_key   = '_customer_user';
	        $meta_value = $userId == 0 ? (int) get_current_user_id() : (int) $userId;
	    } 
	    // Based on billing email (Guest users)
	    else { 
	        $meta_key   = '_billing_email';
	        $meta_value = sanitize_email( $userId );
	    }

	    $count = $wpdb->get_var( $wpdb->prepare("
	        SELECT COUNT(p.ID) FROM {$wpdb->prefix}posts AS p
	        INNER JOIN {$wpdb->prefix}postmeta AS pm ON p.ID = pm.post_id
	        WHERE p.post_type LIKE 'shop_order'
	        AND pm.meta_key = '%s'
	        AND pm.meta_value = %s
	        LIMIT 1
	    ", $meta_key, $meta_value ) );

	    // Return a boolean value based on orders count
	    return $count > 0 ? $count : false;
	}

	protected function get_user_review_count($userId)
	{
		$args = array(
		    'user_id' => $userId,
		    'count'   => true
		);

		$userCommentsCount = get_comments( $args );

		return $userCommentsCount;
	}

	/**
	 * Return the current user role
	 * @author Daniel Barenkamp
	 * @version 1.0.0
	 * @since   1.0.0
	 * @link    https://www.welaunch.io
	 * @return  [type]                       [description]
	 */
	protected function get_user_role($userId)
	{
		$user = get_userdata($userId);
		if(!$user) {
			return false;
		}
		
		$user_roles = $user->roles;
		if(empty($user_roles)) {
			return false;
		}

		$user_role = array_shift($user_roles);

		return $user_role;
	}
}